# buildAndSmoke.tcl: do a clean build and run the smoke test
#
# $Id: buildAndSmoke.tcl,v 1.9 2000/05/02 00:01:27 johnr Exp $
#
# Copyright (c) 1998-2000 The Regents of the University of California.
# All rights reserved. See the file COPYRIGHT for details.
#


### Assumed global variables:
# debug: print debug information if on
# silent: don't print anything if on


#######################################################################
#### buildAndSmoke
#
# Do a clean build and run the smoke test. If the mailto argument
# is supplied, mail the result to that address, otherwise just
# dump it to stdout.
#
# Currently, the smoke test is non-existent.
#
proc buildAndSmoke {DIVA {mailto diva-cvs@gigascale.eecs.berkeley.edu}} {
    global silent debug

    if !$silent {
	puts " ####         Diva build and smoke           ####"
	puts ""
    }
    set dir [file join $DIVA Builds Clean]
    set diva [file join $dir diva]

    # Export the repository into Builds/Clean
    exportDiva $DIVA $dir
    divaConfigure $DIVA $diva

    # Check package directories
    puts "Checking packages"
    set s $silent
    set silent 1
    set t [checkPackageDirs $diva $diva]
    set silent $s
    set badPackages ""
    if { $t != "" } {
	set badPackages "Packages that are not in vars.mk:\n"
	foreach p $t {
	    append badPackages "    [divaPath $diva $p]\n"
	}
	append badPackages \n
    }

    # Check makefiles
    puts "Checking makefiles"
    set s $silent
    set silent 1
    set t [checkMakeDirs $diva $diva]
    set silent $s
    set badMakedirs ""
    if { $t != "" } {
	set badMakedirs "Directories that are not in the makefile:\n"
	foreach d $t {
	    append badMakedirs "    [divaPath $diva $d]\n"
	}
	append badMakedirs \n
    }

    # Go there
    cd $diva

    # Do a "make packages" and get the result
    if !$silent {
	puts "Recompiling"
    }
    catch {exec make -k -w packages >& tmpfile}

    # Do a "make all" and get the result
    ###puts "make all"
    ###catch {exec make all} allResult

    # Do a "make install" and get the result
    ###puts "make install"
    ###catch {exec make install} installResult

    # Construct a diagnosis -- FIXME
    if !$silent {
	puts "Checking compile"
    }
    set errors [parseJavacOutput $DIVA tmpfile]
    if { $errors != "" } {
	set errors "Compile errors:\n$errors"
    }
    file delete tmpfile
    if { $errors == "" && $badPackages == "" && $badMakedirs == "" } {
	set indicator PASSED
	set message "    ### Build and smoke PASSED. Congratulations! ###"
    } else {
	set indicator FAILED
	set message "    ### Build and smoke FAILED. Please fix me!!! ###\n"
	append message "    ### The errors are:\n\n"
	append message $badPackages
	append message $badMakedirs
	append message $errors
	append message "    ### Build and smoke FAILED. Please fix me!!! ###"
    }
    # Dump to stdout
    puts $message

    # Mail results if possible
    if { $mailto != "" } {
	# puts "Mailing result to $mailto"
	set fd [open tmpfile "w"]
	puts $fd $message
	close $fd

	if [catch {
	    exec mailx -s "\[DIVA-BUILD\] Build and smoke $indicator" \
		    $mailto < tmpfile
	}] {
	    puts "Cannot mail message:\n$message"
	}
    }
}







