# buildSnapshot.tcl: build a snapshot release of Diva
#
# $Id: buildCommon.tcl,v 1.4 2000/05/02 00:01:27 johnr Exp $
#
# Copyright (c) 1998-2000 The Regents of the University of California.
# All rights reserved. See the file COPYRIGHT for details.
#

### Assumed global variables:
# debug: print debug information if on
# silent: don't print anything if on


#######################################################################
#### _buildCommon
#
# Build a release of Diva in the current directory. This
# is a helper proc for buildSnapshot, that requires the first argument
# to be the root of an already-exported Diva tree and the second
# argument to be a string suffix identifying the files (either
# DayMonthYear for snapshots or XX.YY for releases).  The third argument
# is a list of packages to be released/documented/etc.
#
# This will generate the following files:
#    diva-suffix.tar.gz  - The UNIX release
#    diva-suffix.zip     - The Windows release
#    diva-suffix.jar     - The JAR file
#    diva-suffix.jar.zip - The Zipped JAR file (win)
#    diva-suffix-docs/   - The online documentation
#
proc _buildCommon {DIVA suffix packages} {
    global silent debug

    cd $DIVA
    if [file exists CVS] {
	error "Diva root \"$DIVA\" must not be under CVS control\n \
		to make it into a snapshot release.\n\n \
		Use \"cvs export -D today diva\" to export Diva first."
    }

    if !$silent {
	puts "Making snapshot \"$suffix\"in \"$DIVA\"..."
    }

    #
    # Don't distribute the web documentation
    #
    file delete -force [file join $DIVA web]

    #
    # Clean up the junk in the tree
    #
    if !$silent {
	puts "Cleaning tree"
    }
    filterJunk $DIVA
    eval filterPackages $DIVA $packages
    eval filterDocs $DIVA $packages

    #
    # Put copyright notices in every directory
    #
    addCopyrights $DIVA $DIVA


    #
    # Create source release files
    #
    cd $DIVA
    cd ..
    file delete diva-$suffix.tar diva-$suffix.tar.gz diva-$suffix.zip 

    puts "tar -cf diva-$suffix.tar diva"
    exec tar -cf diva-$suffix.tar diva

    puts "gzip diva-$suffix.tar"
    exec gzip diva-$suffix.tar
    
    #
    # Windows sucks
    #
    global tcl_platform  ;# Tcl sucks too...
    if { $tcl_platform(platform) != "windows" } {
	windowWasher $DIVA diva-$suffix.zip
    } else {
	puts "zip -q -r -n .jpg:.gif diva-$suffix diva"
	exec zip -q -r -n .jpg:.gif diva-$suffix diva
    }

    #
    # Build! Don't catch it -- if it fails, we want this whole
    # thing to fail
    #
    cd $DIVA
    if !$silent {
	puts "exec make packages"
    }
    set failed [catch {exec make packages} result]
    if $failed {
	puts "ERROR exec make packages failed ERROR\n$result"
    }
    if !$silent {
	puts $result
    }
    eval buildJar $DIVA $packages

    cd $DIVA
    exec zip diva.jar.zip diva.jar

    #
    # Put these in the build directory for now. MMS
    #
    puts "Moving JAR files to the build directory..."
    file rename -force diva.jar [file join .. diva-$suffix.jar]
    file rename -force diva.jar.zip [file join .. diva-$suffix.jar.zip]

    #
    # Generate the API docs
    #
    cd [file join $DIVA doc api]
    eval buildAPI $DIVA $packages
    
    #
    # Move the docs up to the top level and trash the rest of it
    #
    cd $DIVA
    file rename doc ../temp
    cd ..
    file delete -force diva
    if [file exists diva-$suffix] {
	file delete -force diva-$suffix
    }
    file rename temp diva-$suffix

    file rename -force diva-$suffix.jar diva-$suffix
    file rename -force diva-$suffix.jar.zip diva-$suffix
    file rename -force diva-$suffix.tar.gz diva-$suffix
    file rename -force diva-$suffix.zip diva-$suffix
}







