# parseOutput.tcl: parse output from various programs
#
# $Id: parseOutput.tcl,v 1.3 2000/05/02 00:01:29 johnr Exp $
#
# Copyright (c) 1998-2000 The Regents of the University of California.
# All rights reserved. See the file COPYRIGHT for details.
#


### Assumed global variables:
# debug: print debug information if on
# silent: don't print anything if on


#######################################################################
#### parseJavacOutput
# Parse the output from a javac make. The output should have been
# generated using the -w and -k flags to make, and redirected to
# a file with &gt;&amp;. Return the null
# string if there were errors, otherwise return the errors.
#
proc parseJavacOutput {DIVA file} {
    global debug silent

    set allerrors ""
    set fd [open $file]

    # Read a line at a time
    while { ![eof $fd] } {
	set line [gets $fd]
	if { [regexp "^make\[^:\]*: Entering directory `(\[^\n\]*\)'" \
		$line _ directory] } {
	    # Parse for errors
	    set errors ""
	    while { ![eof $fd] } {
		set line [gets $fd]
		if { [regexp "^make\[^:\]*: Leaving directory" $line] } {
		    break
		} elseif { [regexp {^make[^:]*: \*\*\* \[.*\.class\]} $line] } {
		    # Ignore these, we already saw the errors
		} elseif { [regexp "^make\[^:\]*: \\*\\*\\* (.*)$" $line _ err] } {
		    append errors "    *** $err\n"
		} elseif { [regexp "^(\[^:\]*):(\[0-9\]*): (.*)$" \
			$line _ filename linenum err] } {
		    append errors "    $filename:$linenum:\n"
		    append errors "        $err\n"
		    append errors "        [gets $fd]\n"
		    append errors "        [gets $fd]\n"
		}
	    }
	    if { $errors != "" } {
		append allerrors "[divaPath $DIVA $directory]:\n"
		append allerrors "$errors\n"
	    }
	}
    }
    close $fd
    return $allerrors
}






