/*
 * $Id: SketchEvent.java,v 1.4 2000/05/02 00:44:45 johnr Exp $
 *
 * Copyright (c) 1998-2000 The Regents of the University of California.
 * All rights reserved. See the file COPYRIGHT for details.
 */
package diva.sketch;

/**
 * A SketchEvent is generated by sketch model when the model has been
 * modified.  It contains a symbol which is either added, modified, or
 * deleted from the model.
 *
 * @see SketchListener
 * @author Michael Shilman  (michaels@eecs.berkeley.edu)
 * @author Heloise Hse      (hwawen@eecs.berkeley.edu)
 * @version $Revision: 1.4 $
 * @rating Red
 */
public class SketchEvent extends java.util.EventObject {
    /**
     * Symbol added event type.
     */
    public static final int SYMBOL_ADDED    = 0;

    /**
     * Symbol removed event type.
     */
    public static final int SYMBOL_REMOVED  = 1;

    /**
     * Symbol modified event type.
     */
    public static final int SYMBOL_MODIFIED = 2;

    /**
     * Event type ID.
     * @serial
     */
    private int _id;

    /**
     * Create a SketchEvent object with the specified
     * event type and the symbol that initiates this
     * event.
     */
    public SketchEvent(int id, Object source) {
        super(source);
        _id = id;
    }

    /**
     * Return the event type id.
     */
    public int getID() {
        return _id;
    }

    /**
     * Return the symbol that initiates this event.
     */
    public Symbol getSymbol() {
        return (Symbol)getSource();
    }

    /**
     * Return a string representation of the ID.
     */
    private String idToString() {
        switch(getID()) {
            case SYMBOL_ADDED:
                return "SYMBOL_ADDED";
            case SYMBOL_REMOVED:
                return "SYMBOL_REMOVED";
            case SYMBOL_MODIFIED:
                return "SYMBOL_MODIFIED";
            default:
                return "Invalid event ID";
        }
    }

    /**
     * String representation of the event.
     */
    public String toString() {
        return "SketchEvent[" + idToString() + ", " + getSource() + "]";
    }
}

